@@ -11,9 +11,7 @@
 #include <sys/sysctl.h>
 #include <kvm.h>
 
-#include "../../_psutil_common.h"
-#include "../../_psutil_posix.h"
-#include "proc.h"
+#include "../../arch/all/init.h"
 
 
 #define PSUTIL_KPT2DOUBLE(t) (t ## _sec + t ## _usec / 1000000.0)
@@ -26,11 +24,11 @@
 
 
 int
-psutil_kinfo_proc(pid_t pid, kinfo_proc *proc) {
+psutil_kinfo_proc(pid_t pid, struct kinfo_proc2 *proc) {
     // Fills a kinfo_proc struct based on process pid.
     int ret;
     int mib[6];
-    size_t size = sizeof(kinfo_proc);
+    size_t size = sizeof(struct kinfo_proc2);
 
     mib[0] = CTL_KERN;
     mib[1] = KERN_PROC2;
@@ -53,38 +51,7 @@ psutil_kinfo_proc(pid_t pid, kinfo_proc *proc) {
 }
 
 
-struct kinfo_file *
-kinfo_getfile(pid_t pid, int* cnt) {
-    // Mimic's FreeBSD kinfo_file call, taking a pid and a ptr to an
-    // int as arg and returns an array with cnt struct kinfo_file.
-    int mib[6];
-    size_t len;
-    struct kinfo_file* kf;
-    mib[0] = CTL_KERN;
-    mib[1] = KERN_FILE2;
-    mib[2] = KERN_FILE_BYPID;
-    mib[3] = (int) pid;
-    mib[4] = sizeof(struct kinfo_file);
-    mib[5] = 0;
 
-    // get the size of what would be returned
-    if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0) {
-        PyErr_SetFromErrno(PyExc_OSError);
-        return NULL;
-    }
-    if ((kf = malloc(len)) == NULL) {
-        PyErr_NoMemory();
-        return NULL;
-    }
-    mib[5] = (int)(len / sizeof(struct kinfo_file));
-    if (sysctl(mib, 6, kf, &len, NULL, 0) < 0) {
-        PyErr_SetFromErrno(PyExc_OSError);
-        return NULL;
-    }
-
-    *cnt = (int)(len / sizeof(struct kinfo_file));
-    return kf;
-}
 
 PyObject *
 psutil_proc_cwd(PyObject *self, PyObject *args) {
@@ -187,9 +154,9 @@ psutil_proc_exe(PyObject *self, PyObject *args) {
 
 PyObject *
 psutil_proc_num_threads(PyObject *self, PyObject *args) {
-    // Return number of threads used by process as a Python integer.
     long pid;
-    kinfo_proc kp;
+    struct kinfo_proc2 kp;
+
     if (! PyArg_ParseTuple(args, _Py_PARSE_PID, &pid))
         return NULL;
     if (psutil_kinfo_proc(pid, &kp) == -1)
@@ -220,6 +187,7 @@ psutil_proc_threads(PyObject *self, PyObject *args) {
     mib[3] = sizeof(struct kinfo_lwp);
     mib[4] = 0;
 
+    // first query size
     st = sysctl(mib, 5, NULL, &size, NULL, 0);
     if (st == -1) {
         PyErr_SetFromErrno(PyExc_OSError);
@@ -230,16 +198,11 @@ psutil_proc_threads(PyObject *self, PyObject *args) {
         goto error;
     }
 
+    // set slot count for NetBSD KERN_LWP
     mib[4] = size / sizeof(size_t);
-    kl = malloc(size);
-    if (kl == NULL) {
-        PyErr_NoMemory();
-        goto error;
-    }
 
-    st = sysctl(mib, 5, kl, &size, NULL, 0);
-    if (st == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
+    if (psutil_sysctl_malloc(
+            mib, __arraycount(mib), (char **)&kl, &size) != 0) {
         goto error;
     }
     if (size == 0) {
@@ -249,20 +212,22 @@ psutil_proc_threads(PyObject *self, PyObject *args) {
 
     nlwps = (int)(size / sizeof(struct kinfo_lwp));
     for (i = 0; i < nlwps; i++) {
-        if ((&kl[i])->l_stat == LSIDL || (&kl[i])->l_stat == LSZOMB)
+        if (kl[i].l_stat == LSIDL || kl[i].l_stat == LSZOMB)
             continue;
-        // XXX: we return 2 "user" times because the struct does not provide
-        // any "system" time.
-        py_tuple = Py_BuildValue("idd",
-                                 (&kl[i])->l_lid,
-                                 PSUTIL_KPT2DOUBLE((&kl[i])->l_rtime),
-                                 PSUTIL_KPT2DOUBLE((&kl[i])->l_rtime));
+        // XXX: return 2 "user" times, no "system" time available
+        py_tuple = Py_BuildValue(
+            "idd",
+            kl[i].l_lid,
+            PSUTIL_KPT2DOUBLE(kl[i].l_rtime),
+            PSUTIL_KPT2DOUBLE(kl[i].l_rtime)
+        );
         if (py_tuple == NULL)
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
         Py_DECREF(py_tuple);
     }
+
     free(kl);
     return py_retlist;
 
@@ -276,53 +241,42 @@ error:
 
 
 int
-psutil_get_proc_list(kinfo_proc **procList, size_t *procCount) {
-    // Returns a list of all BSD processes on the system.  This routine
-    // allocates the list and puts it in *procList and a count of the
-    // number of entries in *procCount.  You are responsible for freeing
-    // this list (use "free" from System framework).
-    // On success, the function returns 0.
-    // On error, the function returns a BSD errno value.
-    kinfo_proc *result;
+_psutil_pids(struct kinfo_proc2 **proc_list, size_t *proc_count) {
+    struct kinfo_proc2 *result;
     // Declaring name as const requires us to cast it when passing it to
     // sysctl because the prototype doesn't include the const modifier.
     char errbuf[_POSIX2_LINE_MAX];
     int cnt;
     kvm_t *kd;
-
-    assert( procList != NULL);
-    assert(*procList == NULL);
-    assert(procCount != NULL);
+    size_t mlen;
 
     kd = kvm_openfiles(NULL, NULL, NULL, KVM_NO_FILES, errbuf);
-
     if (kd == NULL) {
         PyErr_Format(
-            PyExc_RuntimeError, "kvm_openfiles() syscall failed: %s", errbuf);
+            PyExc_RuntimeError, "kvm_openfiles() syscall failed: %s", errbuf
+        );
         return 1;
     }
 
-    result = kvm_getproc2(kd, KERN_PROC_ALL, 0, sizeof(kinfo_proc), &cnt);
+    result = kvm_getproc2(kd, KERN_PROC_ALL, 0, sizeof(struct kinfo_proc2), &cnt);
     if (result == NULL) {
         PyErr_Format(PyExc_RuntimeError, "kvm_getproc2() syscall failed");
         kvm_close(kd);
         return 1;
     }
 
-    *procCount = (size_t)cnt;
+    *proc_count = (size_t)cnt;
 
-    size_t mlen = cnt * sizeof(kinfo_proc);
-
-    if ((*procList = malloc(mlen)) == NULL) {
+    mlen = cnt * sizeof(struct kinfo_proc2);
+    if ((*proc_list = malloc(mlen)) == NULL) {
         PyErr_NoMemory();
         kvm_close(kd);
         return 1;
     }
 
-    memcpy(*procList, result, mlen);
-    assert(*procList != NULL);
+    memcpy(*proc_list, result, mlen);
+    assert(*proc_list != NULL);
     kvm_close(kd);
-
     return 0;
 }
 
@@ -332,7 +286,7 @@ psutil_proc_cmdline(PyObject *self, PyObject *args) {
     pid_t pid;
     int mib[4];
     int st;
-    int attempt;
+    int attempt = 0;
     int max_attempts = 50;
     size_t len = 0;
     size_t pos = 0;
@@ -350,21 +304,9 @@ psutil_proc_cmdline(PyObject *self, PyObject *args) {
     mib[2] = pid;
     mib[3] = KERN_PROC_ARGV;
 
-    st = sysctl(mib, __arraycount(mib), NULL, &len, NULL, 0);
-    if (st == -1) {
-        PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_ARGV) get size");
-        goto error;
-    }
-
-    procargs = (char *)malloc(len);
-    if (procargs == NULL) {
-        PyErr_NoMemory();
-        goto error;
-    }
-
     while (1) {
-        st = sysctl(mib, __arraycount(mib), procargs, &len, NULL, 0);
-        if (st == -1) {
+        if (psutil_sysctl_malloc(
+                mib, __arraycount(mib), (char **)&procargs, &len) != 0) {
             if (errno == EBUSY) {
                 // Usually happens with TestProcess.test_long_cmdline. See:
                 // https://github.com/giampaolo/psutil/issues/2250
@@ -377,14 +319,10 @@ psutil_proc_cmdline(PyObject *self, PyObject *args) {
                     psutil_debug(
                         "proc %zu cmdline(): return [] due to EBUSY", pid
                     );
-                    free(procargs);
                     return py_retlist;
                 }
             }
-            else {
-                PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_ARGV)");
-                goto error;
-            }
+            goto error;
         }
         break;
     }
